import com.thoughtworks.xstream.XStream;

import flex.messaging.io.SerializationContext;
import flex.messaging.io.amf.ActionContext;
import flex.messaging.io.amf.ActionMessage;
import flex.messaging.io.amf.AmfMessageDeserializer;
import flex.messaging.io.amf.AmfMessageSerializer;
import flex.messaging.io.amf.AmfTrace;

import java.beans.XMLEncoder;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import flex.messaging.io.ClassAliasRegistry ;

public class JserEditor
{
  private String XMLEditorFullPath;
  private String NormalEditor;
  private String FileFullPath;
  private byte[] Headers;
  private byte[] Payload;
  private byte[] message;
  private String strMessage;

  JserEditor(String Editor, String NormalEditorfile, String FileName)
  {
    this.XMLEditorFullPath = Editor;
    this.NormalEditor = NormalEditorfile;
    this.FileFullPath = FileName;
    if (this.FileFullPath != "")
    {
      this.message = readfileToPayload(this.FileFullPath);
      this.strMessage = new String(this.message);
      this.Headers = this.strMessage.split("\\r\\n\\r\\n")[0].getBytes();
      int startPayload = this.strMessage.indexOf("\r\n\r\n");
      int startPayLoadIndex = startPayload + 4;
      byte[] byteOrigMessageContent = new byte[this.message.length - startPayload - 4];
      int i = 0;
      int j = 0;
      try
      {
        i = startPayLoadIndex;
        for (j = 0; j < byteOrigMessageContent.length; j++)
        {
          byteOrigMessageContent[j] = this.message[i];
          i++;
        }

      }
      catch (Exception e)
      {
        System.out.println("Exception" + e);
      }
      this.Payload = byteOrigMessageContent;
    }
  }

  private byte[] readfileToPayload(String file)
  {
    try
    {
      File input = new File(file);
      FileInputStream fis = new FileInputStream(input);
      ByteArrayOutputStream baos = new ByteArrayOutputStream();
      int data;
      while ((data = fis.read()) != -1)
      {
        //int data;
        baos.write(data);
      }baos.close();
      return baos.toByteArray();
    }
    catch (IOException ex)
    {
      ex.printStackTrace();
    }
    return null;
  }

  private void execEditor()
  {
    Runtime rt = Runtime.getRuntime();
    try
    {
      String editor = this.NormalEditor;
      if ((this.XMLEditorFullPath != "") && (isJSER()))
        editor = this.XMLEditorFullPath;
      Process ps = rt.exec(editor + " " + this.FileFullPath);
      ps.waitFor();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    catch (InterruptedException e)
    {
      e.printStackTrace();
    }
  }

  public static void main(String[] args) throws ClassNotFoundException
  {
    if (args.length > 2)
    {
      JserEditor EE = new JserEditor(args[0], args[1], args[2]);
      EE.Manipulate();
    }
    else {
      System.out.println("Usage: Jser.jar \"XML Editor Path\" \"Normal Editor path\" \"File to Edit");
    }
  }

  private void copyFiles(String src, String dst)
  {
    try
    {
      FileWriter fw = new FileWriter(dst);
      FileReader fr = new FileReader(src);
      BufferedWriter bw = new BufferedWriter(fw);
      BufferedReader br = new BufferedReader(fr);
      for (String line = ""; (line = br.readLine()) != null; ) {
        bw.append(line + '\n');
      }
      bw.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
  }

  private void writeFile()
  {
    try
    {
      File output = new File(this.FileFullPath);
      FileOutputStream fos = new FileOutputStream(output);
      ByteArrayInputStream inp = new ByteArrayInputStream(this.message);
      int data;
      while ((data = inp.read()) != -1)
      {
        //int data;
        fos.write(data);
      }fos.close();
    }
    catch (IOException ex)
    {
      ex.printStackTrace();
    }
  }

  private boolean isJSER()
  {
    return (this.Payload.length > 2) && ((char)(this.Payload[0] & 0xFF) == '¬') && ((char)(this.Payload[1] & 0xFF) == 'í');
  }
 
  private boolean isAMF()
  {
      return  ((new String(Headers).indexOf("application/x-amf"))!=-1);
  }

  private void Manipulate() throws ClassNotFoundException
  {
    Object O = new Object();
    String originalFile = this.FileFullPath;
    String fileToUse = originalFile;
    if (new String(this.Headers).contains("Belch: XML"))
    {
      String xmlPayload = new String(this.Payload);
      XStream xstream = new XStream();
      O = xstream.fromXML(xmlPayload);
      if (isJSER())
    	  this.Payload = WriteObjectToPayload(O);
      else if (isAMF())
    	  this.Payload = WriteAMFObjectToPayload(O);
      update();
      writeFile();
    }
    else if (isJSER()||isAMF())
    {
      try {
        if (isJSER())
        	O = ReadObjectFromPayload(this.Payload);
        else if (isAMF())
        	O = ReadAMFObjectFromPayload(this.Payload);

        
      }
      catch (Exception cnfE)
      {
        System.out.println("Error, Class not found for " + cnfE.getCause());
      }
      String name = O.getClass().getSimpleName();
      System.out.println("got a " + name + " Class");
      Object2XMLfile(O, fileToUse);
      this.FileFullPath = fileToUse;
    }
    else
    {
      execEditor();
    }
  }

  public static byte[] toByte(String hexString) {
    int len = hexString.length() / 2;
    byte[] result = new byte[len];
    for (int i = 0; i < len; i++)
      result[i] = Integer.valueOf(hexString.substring(2 * i, 2 * i + 2), 16).byteValue();
    return result;
  }

  private void update()
  {
    String msgHeaders = new String(this.Headers);
    String Apart = msgHeaders.split("Content-Length: ")[0];
    String Bpart = msgHeaders.split("Content-Length: ")[1];
    int contentIndx = Bpart.indexOf("\r\n");
    Bpart = Bpart.substring(contentIndx);
    
    //String Apart = Headers.split("Content-Length: ")[0];

    //String Bpart = Headers.split("Content-Length: ")[1].split("\r\n")[1];
    String Combined = Apart + "Content-Length: " + this.Payload.length + new StringBuilder().append(Bpart).append("\r\n\r\n").toString();
    if (isJSER()||isAMF())
      Combined = Combined.split("Belch: XML\r\n")[0] + Combined.split("Belch: XML\r\n")[1];
    byte[] NewMessage = new byte[this.Payload.length + Combined.length()];
    byte[] headerByte = Combined.getBytes();
	int i=0;
    for (i = 0; i < headerByte.length; i++) {
      NewMessage[i] = headerByte[i];
    }
    for (int j = 0; j < this.Payload.length; )
    {
      NewMessage[i] = this.Payload[j];
      j++;
      i++;
    }

    this.message = NewMessage;
    this.Headers = headerByte;
  }

  private void Object2XMLfile(Object O, String fileName)
  {
    StringBuffer XML = new StringBuffer();
    XStream xstream = new XStream();
    XML.append(xstream.toXML(O));
    //
    

    
    //
    try
    {
      
      FileWriter tempXML = new FileWriter(fileName);
      BufferedWriter bw = new BufferedWriter(tempXML);
      String msgStr = new String(this.message);
      
      String msgHeaders = msgStr.split("\\r\\n\\r\\n")[0];
      
      String newMessageA = msgHeaders.split("Content-Length: ")[0];
      String newMessageB = msgHeaders.split("Content-Length: ")[1];
      int contentIndx = newMessageB.indexOf("\r\n");
      if (contentIndx>-1)
    	  newMessageB = newMessageB.substring(contentIndx);
      else
    	  newMessageB = "";
     //newMessageB = newMessageB.substring(contentIndx);
      
                                                            //.split("\\r\\n")[1].split("\\r\\n\\r\\n")[0];

      bw.write(newMessageA + "Content-Length: " + XML.length()  + newMessageB);
      bw.write("\r\nBelch: XML");
      bw.write("\r\n\r\n");

      bw.write(XML.toString());
      bw.close();
      tempXML.close();
    }
    catch (FileNotFoundException e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
  }

  private Object ReadObjectFromPayload(byte[] payload)
  {
    Object O = new Object();
    try
    {
      ByteArrayInputStream bis = new ByteArrayInputStream(payload);
      ObjectInputStream ois = new ObjectInputStream(bis);
      O = ois.readObject();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    catch (ClassNotFoundException e)
    {
      e.printStackTrace();
    }
    return O;
  }

  private Object ReadAMFObjectFromPayload(byte payload[])
  {
	    ClassAliasRegistry.getRegistry().registerAlias("DSK", "flex.messaging.messages.AcknowledgeMessageExt");  	
	  	ClassAliasRegistry.getRegistry().registerAlias("DSC", "flex.messaging.messages.CommandMessageExt");
		ClassAliasRegistry.getRegistry().registerAlias("DSA", "flex.messaging.messages.AsyncMessageExt");    	
  	SerializationContext serialContext = SerializationContext.getSerializationContext();
      serialContext.instantiateTypes = false;
      AmfTrace trace = null;
      if(false)
         trace = new AmfTrace();
      AmfMessageDeserializer amfder = new AmfMessageDeserializer();
      amfder.initialize(serialContext, new ByteArrayInputStream(payload), trace);
      ActionMessage AMF = new ActionMessage();
      ActionContext messageContext = new ActionContext();
      try {
			amfder.readMessage(AMF, messageContext);
		} catch (ClassNotFoundException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
			
		}
  	
  	
      catch(Exception e)
      {
          e.printStackTrace();
          return AMF;
      }
      return AMF;
  }
  

  private byte[] WriteObjectToPayload(Object O)
  {
    try
    {
      ByteArrayOutputStream baos = new ByteArrayOutputStream();
      ObjectOutputStream oos = new ObjectOutputStream(baos);
      oos.writeObject(O);
      return baos.toByteArray();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return new byte[0];
  }
  private byte[] WriteAMFObjectToPayload(Object O)
  {
      try
      {
    	    ClassAliasRegistry.getRegistry().registerAlias("DSK", "flex.messaging.messages.AcknowledgeMessageExt");  	
    	  	ClassAliasRegistry.getRegistry().registerAlias("DSC", "flex.messaging.messages.CommandMessageExt");
    		ClassAliasRegistry.getRegistry().registerAlias("DSA", "flex.messaging.messages.AsyncMessageExt");
    	  SerializationContext serialContext = SerializationContext.getSerializationContext();
      	ByteArrayOutputStream baos = new ByteArrayOutputStream();
      	serialContext.instantiateTypes = false;
          AmfTrace trace = null;
          if(false)
             trace = new AmfTrace();
          AmfMessageSerializer amfser = new AmfMessageSerializer();
          amfser.initialize(serialContext, baos, trace);
          ActionMessage AMF = (ActionMessage)O;
                      
          try {
  			amfser.writeMessage(AMF);
  		} catch (IOException e1) {
  			// TODO Auto-generated catch block
  			e1.printStackTrace();
  		}      	
      	
      	
          return baos.toByteArray();
      }
      catch(Exception e)
      {
          e.printStackTrace();
      }
      return new byte[0];
  }
  private Object XMLFile2Object(String filename)
  {
    Object O = new Object();
    try
    {
      FileReader file = new FileReader(filename);
      BufferedReader buff = new BufferedReader(file);
      StringBuffer sb = new StringBuffer();
      for (String s = ""; (s = buff.readLine()) != null; )
        sb.append(s + "\r\n");
      XStream xstream = new XStream();

      this.Headers = sb.toString().split("\r\n\r\n")[0].getBytes();
      O = xstream.fromXML(sb.toString().split("\r\n\r\n")[1]);
    }
    catch (FileNotFoundException e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return O;
  }
}